/* Cdigo para el clculo de la conductividad trmica en cadenas
   desordenadas en funcin de la temperatura.
   Francisco Domnguez-Adame.                                         */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>

#define N  500          /* Nmero de tomos de la cadena              */
#define NP  50          /* Nmero de promedios                        */
#define MA   1.0        /* Masa de los tomos A                       */
#define MB   1.5        /* Masa de los tomos B                       */
#define C    0.05       /* Concentracin de tomos B                  */
#define LMIN 0.0        /* Frecuencia mnima                          */
#define LMAX 4.0        /* Frecuencia mxima                          */
#define NL 2000         /* Nmero de frecuencias                      */
#define TMIN 0.01       /* Temperatura mnima                         */
#define TMAX 1.01       /* Temperatura mxima                         */
#define DT   0.002      /* Paso en temperaturas                       */
#define PI   3.14159652
#define FILENAME  "DATOS.DAT"  /* Archivo de datos                    */
            /* Generador de numeros aleatorios en el intervalo [0,1]  */
#define RAN() ((double)rand()/(double)(RAND_MAX))

double alfa[N+1];       /* Vector con las masas reducidas             */
double tau[NL+1];       /* Vector con el coeficiente transmisin      */

void red (void){        /* Genera una cadena desordenada              */
    int i;
    for(i=1;i<=N;i++){
       if(RAN()>=C) alfa[i]=1.;
       else alfa[i]=MB/MA;
    }
}

void transmision(void) {/* Calcula el coeficiente de transmisin      */
   double t11_0,t11_1,t11,t12_0,t12_1,t12,t21,t22,l,dl,aux;
   int i=0,j;
   dl=(double)(LMAX-LMIN)/NL;
   for(l=LMIN+dl;l<=LMAX;l+=dl) {  /* Bucle en frecuencias            */
      i++;
      t11_0=1.;                    /* Condiciones iniciales           */
      t12_0=0.;
      t11_1=2.-l*alfa[1];
      t12_1=-1.;
      for(j=2;j<=N;j++) {          /* Bucle en la cadena              */
         t11=(2.-l*alfa[j])*t11_1-t11_0;
         t12=(2.-l*alfa[j])*t12_1-t12_0;
         if(j==N-1) {
            t21=t11;
            t22=t12;
         }
         t11_0=t11_1;
         t12_0=t12_1;
         t11_1=t11;
         t12_1=t12;
      }
      aux=(t11-t22)*(t11-t22)+(t12-t21)*(t12-t21);
      aux+=(2.-l)*(t11-t22)*(t12-t21);
      aux+=l*(4.-l)*t11*t22;
      tau[i]+=l*(4.-l)/aux/(double)NP;
   }
}

double funcion(double dl,double l,double t) {
   double aux;
   aux=exp(sqrt(l)/t/PI);
   return (dl*aux*sqrt(l)/(aux-1.)/(aux-1.));
}

void main(void) {
   int i;
   double l,dl=(double)(LMAX-LMIN)/NL,temp,dt=DT,sum1,sum2,aux1,aux2;
   FILE *out;
   time_t t;
   srand ((unsigned) time(&t));
   out=fopen(FILENAME,"w");
   for(i=0;i<=NL;i++) tau[i]=0.;/* Inicializa los vectores            */
   for(i=1;i<=NP;i++) {         /* Comienzan los promedios            */
      red();
      transmision();
   }
   for(temp=TMIN;temp<TMAX;temp+=dt) {
      aux1=funcion(dl,LMIN+dl,temp);
      aux2=funcion(dl,LMAX,temp);
      sum1=0.5*(aux1+aux2);
      sum2=0.5*dl*(tau[1]*aux1+tau[NL]*aux2);
      i=1;
      for(l=LMIN+2.*dl;l<LMAX;l+=dl) {
         i++;
         aux1=funcion(dl,l,t);
         aux2=tau[i]*aux1;
         sum1+=aux1;
         sum2+=aux2;
      }
      fprintf(out,"%4.3lf   ",temp);
      fprintf(out,"%8.6lf\n",1./(temp*temp*(1./sum2-1./sum1)));
   }
   fclose(out);
}

