/* Cdigo para el clculo de las relaciones de dispersin
   exacta y aproximada en el modelo de Kronig-Penney.
   Francisco Domnguez-Adame.                                         */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define E0    -13.60         /* Energa del nivel atmico en eV       */
#define D       3.00         /* Distancia interatmica en A           */
#define EMIN  -13.75         /* Energa mnima en eV                  */
#define EMAX  -13.45         /* Energa mxima en eV                  */
#define ESTEP   0.0002       /* Paso en energa en eV                 */
#define PI 3.1412592654
#define FILENAME1  "DATOS1.DAT" /* Archivo resultados exactos         */
#define FILENAME2  "DATOS2.DAT" /* Archivo resultados aproximados     */

void main(void) {
   float e,p,aux;
   FILE *out1, *out2;
   out1=fopen(FILENAME1,"w");
   out2=fopen(FILENAME2,"w");
   for(e=EMIN;e<=EMAX;e+=ESTEP) {
      aux=2.*D*sqrt(e/E0);
      p=cosh(aux)-2.*D*sinh(aux)/aux;  /* Valor exacto                */
      if(fabs(p)<1.) fprintf(out1,"%7.5f   %7.5f\n",acos(p)/PI,e);
      p=0.25*(e/E0-1.)*exp(2.*D);      /* Valor aproximado            */
      if(fabs(p)<1.) fprintf(out2,"%7.5f   %7.5f\n",acos(p)/PI,e);
   }
   fclose(out1);
   fclose(out2);       /* Cierra los ficheros de datos                */
}
