/* Cdigo para el clculo de la estructura de bandas en el potencial
   de Mathieu utilizando el desarrollo en ondas planas.
   Francisco Domnguez-Adame.                                         */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define N     7         /* Nmero de ondas planas (impar)             */
#define NB    2         /* Nmero de bandas a estudiar (NB<=N)        */
#define V0    0.2       /* Constante del potencial                    */
#define DK    0.001     /* Incremento del vector recproco            */
#define EPS   1e-8      /* Precisin en tridiagonal                   */
#define NMAX  30        /* Nmero mximo de iteraciones en tridiagonal*/
#define FILENAME "DATOS.DAT"  /* Archivo de datos                     */
#define SIGN(a) ((a)<0 ? -1 : 1)

double d[N+1];  /* Almacena los elementos de la diagonal al inicio
                   y devuelve los autovalores de la matriz            */
double e[N+1];  /* Almacena los elementos de la no-diagonal al
                   inicio y desaparece en la salida                   */

void elementos(double *d,double *e,double k) {
    int i,n=(N-1)/2;
    double aux;
    for(i=1;i<=N;i++)  {
       aux=k+2.*(double)(i-1-n);
       d[i]=aux*aux;
       e[i]=0.5*V0;
    }
    e[N]=0.;
}

void tridiagonal(double d[N],double e[N],int n) {
   int i,j,iter=0,div;
   double c,s,t,ct2,p,se,ce,aux;
   for(j=1;j<=n;j++) {
      for(div=j;div<=n;div++) if(fabs(e[div])<EPS) break;
      while(div!=j) {
         if(div==j+1) {
            ct2=0.5*(d[j]-d[j+1])/e[j];
            t=SIGN(ct2)/(fabs(ct2)+sqrt(ct2*ct2+1.));
            c=1./sqrt(t*t+1.);
            s=t*c;
            d[j]+=t*e[j];
            d[j+1]-=t*e[j];
            e[j]=0.;
            j++;
         } else {
            if(iter++==NMAX) {
               printf("Demasiadas iteraciones en tridiagonal\n");
               exit(1);
            }
            ct2=d[div]-d[j];
            s=c=1.;
            p=0.;
            for(i=div-1;i>=j;i--) {
               se=s*e[i];
               ce=c*e[i];
               if(fabs(se)>=fabs(ct2)) {
                   c=ct2/se;
                   aux=sqrt(c*c+1.);
                   e[i+1]=se*aux;
                   s=1./aux;
                   c*=s;
               } else {
                   s=se/ct2;
                   aux=sqrt(s*s+1.);
                   e[i+1]=ct2*aux;
                   c=1./aux;
                   s*=c;
               }
               ct2=d[i+1]-p;
               aux=(d[i]-ct2)*s+2.*c*ce;
               p=s*aux;
               d[i+1]=ct2+p;
               ct2=c*aux-ce;
            }
            d[j]-=p;
            e[j]=ct2;
            e[div]=0.;
          }
          for(div=j;div<=n;div++) if(fabs(e[div])<EPS) break;
      }
   }
}

void banda(double *d,double *e,double k) {
   double tmp;
   int i,j;
   elementos(d,e,k);
   tridiagonal(d,e,N);
   for(i=1;i<N;i++)          /* Ordena los autovalores                */
      for(j=i+1;j<=N;j++)
         if(d[j]<d[i]) {
            tmp=d[j];
            d[j]=d[i];
            d[i]=tmp;
         }
}

void main(void) {
   double k,dk=DK;
   int i;
   FILE *out;
   out=fopen(FILENAME,"w");
   for(k=0.;k<=1.;k+=dk){
      banda(d,e,k);
      fprintf(out,"%8.6lf   ",k);
      for(i=1;i<=NB;i++) fprintf(out,"%8.6lf   ",d[i]);
      fprintf(out,"\n");
   }
   fclose(out);
}
