/* Cdigo para calcular la densidad de estados en un modelo de
   enlace fuerte de una red cbica simple.
   Francisco Domnguez-Adame.                                         */

#include <stdio.h>
#include <math.h>
#include <stdlib.h>

#define EMIN   0.0    /* Energa mnima                               */
#define EMAX   6.0    /* Energa mxima                               */
#define NE   500      /* Nmero de pasos en energa                   */
#define L     20.0    /* Anchura de cada intervalo de integracin     */
#define EPS    1.0e-5 /* Precisin requerida                          */
#define JMAX 180      /* Nmero mximo de iteraciones                 */
#define FILENAME "DATOS.DAT"  /* Archivo de datos                     */
#define PI 3.141592654

double funcion(float x, float e) {
    double r,aux,f0,theta0;
    if(x<=3.0) {
        aux=x*x/9.;
        r=1.+aux*(-2.2499997+aux*(1.2656208+aux*(-0.3163866+aux*(
          0.0444479+aux*(-0.0039444+aux*0.00021)))));
    } else {
       aux=3./x;
       f0=0.797888+aux*(-0.00000077+aux*(-0.0055274+aux*(-0.00009512+
          aux*(0.00137237+aux*(-0.00072805+aux*0.00014476)))));
       theta0=x-0.785399+aux*(-0.04166397+aux*(-0.00003954+aux*(
          0.00262573+aux*(-0.00054125+aux*(-0.00029333+
          aux*0.00013558)))));
       r=f0*cos(theta0)/sqrt(x);
    }
    return (r*r*r*cos((e-3.)*x));
}

double simpson(float a,float b,float e) {
   int i,j=0,n=1;
   double eps,jn,j2n,sa,sd,sum,c=b-a;
   eps=sa=1.0e30;
   jn=0.5*(funcion(a,e)+funcion(b,e));
   while(eps>EPS) {
      j++;
      if(j==JMAX) {
         printf("Demasiadas iteraciones en simpson\n");
         exit(1);
      }
      sum=0.0;
      for(i=1;i<=n;i++) sum+=funcion(a+c*(float)(2.*(n-i)+1.)/(2.*n),e);
      j2n=jn+sum;
      sd=c*(2.*j2n-jn)/(3.*n);
      eps=fabs(sd/sa-1);
      sa=sd;
      jn=j2n;
      n*=2;
   }
   return sd;
}

void main(void) {
   double e,de=(EMAX-EMIN)/(float)NE,res,aux,eps;
   int j;
   FILE *out;
   out=fopen(FILENAME,"w");
   for(e=EMIN;e<=EMAX;e+=de) {
       j=0;
       eps=1.0e30;
       res=simpson(0.,L,e);
       while(eps>EPS) {
          j++;
          if(j==JMAX) {
             printf("Demasiadas iteraciones en la funcin main\n");
             exit(1);
          }
          aux=simpson(j*L,(j+1)*L,e);
          res+=aux;
          eps=fabs(aux);
       }
       fprintf(out,"%8.6lf   %8.6lf\n",e,res/PI);
   }
   fclose(out);
}
