/* Cdigo para el clculo de la energa de estados superficiales
   mediante un modelo de Kronig-Penney semi-infinito.
   Francisco Domnguez-Adame.                                          */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define P    9.4248     /* Constante de acoplamiento                   */
#define V   12.5        /* Potencial superficial en eV                 */
#define A    3.7        /* Parmetro de red en Amstrongs               */
#define C    0.2625     /* 2m/hbar^{2} ev^{-1}A^{-2}                   */
#define R        3.0    /* Valor inicial de la raz                    */
#define EPS      1e-6   /* Precisin en el valor de la raz            */
#define NMAX    30      /* Nmero mximo de iteraciones                */
#define CELDAS   4      /* Nmero de celdas para calcular P(z)         */
#define DZ       0.01   /* Paso en z en unidades de A                  */
#define FILENAME  "DATOS.DAT"  /* Archivo de datos                     */

double funcion(double r,double r0) {
   double num,den;
   if(r>r0) {
      printf("Error de dominio en funcion\n");
      exit(1);
   }
   num=r/tan(r)+sqrt(r0*r0-r*r)-0.5*r0*r0/P;
   den=1./tan(r)-r/(sin(r)*sin(r))-r/sqrt(r0*r0-r*r);
   return (num/den);
}

void main(void) {
   double r,r0,aux=R,b,energia,eps=1e10,z,dz=DZ,p_z;
   int i;
   FILE *out;
   out=fopen(FILENAME,"w");
   r0=sqrt(C*V)*A;
   while(eps>EPS) {
      i++;
      if(i==NMAX) {             /* Posible mala convergencia           */
         printf("Demasiadas iteraciones en el proceso\n");
         exit(1);
      }
      r=aux-funcion(aux,r0);
      eps=fabs(r-aux);
      aux=r;
   }
   energia=r*r/(C*A*A);
   fprintf(out,"Energa del estado superficial %8.6lf eV\n",energia);
   b=cos(r)+sqrt(r0*r0-r*r)*sin(r)/r;
   if(P>sqrt(r0*r0-r*r))
      fprintf(out,"Se cumple el criterio de existencia\n");
   else
      fprintf(out,"No se cumple el criterio de existencia\n");
   for(z=-1;z<=CELDAS;z+=dz) { /* Coordenada z en unidades de A        */
      aux=floor(z);
      if(z<0) p_z=exp(sqrt(r0*r0-r*r)*z);
      else p_z=pow(b,aux)*(b*sin(r*z-r*aux)-sin(r*z-r*aux-r))/sin(r);
      fprintf(out,"%5.3lf   %8.6lf\n",z,p_z*p_z);
   }
   fclose(out);
}
