/* Cdigo para el clculo del coeficiente de transmisin en
   dispositivos de efecto tnel resonante.
   Francisco Domnguez-Adame.                                         */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define N 1000           /* Nmero de divisiones del intervalo        */
#define C    0.0176      /* 2m^{*}/hbar^2 en eV^{-1}A^{-2}            */
#define E    0.050       /* Energa del electrn en eV                */
#define DE   0.250       /* Altura de la barrera en eV                */
#define L  150.0         /* Anchura del dispositivo en Amstrongs      */
#define D   50.0         /* Anchura del pozo en Amstrongs             */
#define VMIN 0.0         /* Potencial mnimo en voltios               */
#define VMAX 0.15        /* Potencial mximo en voltios               */
#define NV 300           /* Nmero de pasos del potencial             */
#define FILENAME  "DATOS.DAT"   /* Archivo de datos                   */

double t11[N+1],t12[N+1];/* Elementos de matriz de transferencia      */

double potencial(double x,double v) {
    if ((L-D)/2.>x || (L+D)/2.<x) return (DE-v*x/L);
    else return (-v*x/L);
}

void main(void) {
   double q=sqrt(C*E),qprima,alfa,v,dv=(double)(VMAX-VMIN)/NV;
   double deno,tau,cq,cqprima,sq,sqprima,a,b,c,d,s=(double)L/N;
   int i;
   FILE *out;
   out=fopen(FILENAME,"w");
   cq=cos(q*s);
   sq=sin(q*s);
   for(v=VMIN;v<=VMAX;v+=dv) {     /* Bucle en potenciales            */
      alfa=2+C*s*s*(potencial(s,v)-E);
      qprima=sqrt(C*(E+v));
      cqprima=cos(qprima*s);
      sqprima=sin(qprima*s);
      t11[0]=alfa;                 /* Condiciones iniciales           */
      t11[1]=alfa*alfa-1.;
      t12[0]=-1.;
      t12[1]=-alfa;
      for(i=2;i<=N;i++) {
         alfa=2+C*s*s*(potencial(s*i,v)-E);
         t11[i]=alfa*t11[i-1]-t11[i-2];
         t12[i]=alfa*t12[i-1]-t12[i-2];
      }
      a=t11[N];
      b=t12[N];
      c=t11[N-1];
      d=t12[N-1];
      deno=a*a+b*b+c*c+d*d+2.*(a*b+c*d)*cq-2*(a*c+b*d)*cqprima
          -2.*(a*d+b*c)*cq*cqprima+2.*sq*sqprima;
      tau=4.*sq*sqprima/deno;
      fprintf(out,"%8.6lf   %8.6lf\n",v,log10(tau));
   }
   fclose(out);
}
