/* Cdigo para calcular la imanacin en un campo magntico mediante las
   ecuaciones de Bloch. Su emplea el mtodo de Runge-Kutta.
   Francisco Domnguez-Adame                                          */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#define ALFA     0.5           /* Valor del parmetro alfa            */
#define TMAX     2.5           /* Tiempo reducido mximo              */
#define NT    1000             /* Nmero de pasos temporales          */
#define MX0      1.0           /* Imanacin reducida x inicial        */
#define MY0      0.0           /* Imanacin reducida y inicial        */
#define PI       3.141592654
#define FILENAME "DATOS.DAT"

float m[2];               /* Almacena las componentes de la imanacin */
float f[2];               /* Almacena las derivadas de la imanacin   */

void funcion(float *m,float *f) {
   f[0]=(2.*PI/ALFA)*m[1]-m[0];
   f[1]=-(2.*PI/ALFA)*m[0]-m[1];
}

void runge_kutta(float *m,float *f,float h) {
   int i;
   float k2[2],k3[2],k4[2],m_aux[2];
   for(i=0;i<=1;i++) m_aux[i]=m[i]+0.5*h*f[i];
   funcion(m_aux,k2);
   for(i=0;i<=1;i++) m_aux[i]=m[i]+0.5*h*k2[i];
   funcion(m_aux,k3);
   for(i=0;i<=1;i++) m_aux[i]=m[i]+h*k3[i];
   funcion(m_aux,k4);
   for(i=0;i<=1;i++) m[i]+=h*(f[i]+2.*k2[i]+2.*k3[i]+k4[i])/6.;
}

void main(void) {
   int i;
   float dt=TMAX/(float)NT;
   FILE *out;
   out=fopen(FILENAME,"w");
   m[0]=MX0;                  /* Condiciones iniciales                */
   m[1]=MY0;
   funcion(m,f);              /* Derivadas iniciales                  */
   for(i=1;i<NT;i++) {
      runge_kutta(m,f,dt);
      funcion(m,f);           /* Calcula las derivadas en t=TMIN      */
      fprintf(out,"%5.4f   %5.4f   %5.4f",(float)i*dt,m[0],m[1]);
      fprintf(out,"   %5.4f\n",sqrt(m[0]*m[0]+m[1]*m[1]));
   }
   fclose(out);
}
