/* Cdigo para el clculo del intervalo prohibido en un superconductor
   en funcin de la temperatura segn la teora BCS.
   Francisco Domnguez-Adame                                          */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/* Parmetros del sistema                                             */

#define TAUMIN 0.10     /* Temperatura reducida mnima                */
#define TAUMAX 0.99     /* Temperatura reducida mxima                */
#define DTAU   0.005    /* Paso en temperatura reducida               */
#define JMAX 20         /* Nmero mximo de iteraciones               */
#define EPS 1.0e-10     /* Precisin requerida en simpson             */
#define RPS 1.0e-5      /* Precisin requerida en la raz             */
#define FILENAME  "DATOS.DAT"

double funcion(double x,double delta,double tau) {
   double r;
   r=sqrt(x*x+1.);
   r*=exp(1.75*r*delta/tau)+1.;
   return (1./r);
}

double simpson(double delta,double tau,double a,double b) {
   int i,j=0,n=1;
   double eps,jn,j2n,sa,sd,sum,c=b-a;
   eps=sa=1.0e30;
   jn=0.5*(funcion(a,delta,tau)+funcion(b,delta,tau));
   while(eps>EPS) {
      j++;
      if(j==JMAX) {
         printf("Demasiadas iteraciones en simpson\n");
         exit(1);
      }
      sum=0.0;
      for(i=1;i<=n;i++)
         sum+=funcion(a+c*(float)(2.*(n-i)+1.)/(2.*n),delta,tau);
      j2n=jn+sum;
      sd=c*(2.*j2n-jn)/(3.*n);
      eps=fabs(sd/sa-1);
      sa=sd;
      jn=j2n;
      n*=2;
   }
   return sd;
}

double integral(double g,double tau) {
   double aux=1e20,limsup=1,r;
   r=simpson(g,tau,0.,limsup);
   while(fabs(r-aux)>RPS) {
      limsup*=2.;
      aux=r;
      r=simpson(g,tau,0.,limsup);
   }
   return (log(g)+2.*r);
}

void main(void) {
   double gmin,gmax,gmitad,tau,dtau=DTAU;
   double rmin,rmax,rmitad,rps;
   int i;
   FILE *out;
   out=fopen(FILENAME,"w");
   for(tau=TAUMIN;tau<TAUMAX;tau+=dtau) { /* Bucle en temperaturas    */
      rps=1.0e20;
      i=0;
      if(tau<0.5) {
         gmin=0.90;
         gmax=1.00;
      }
      if(tau>=0.5 && tau<0.6) {
         gmin=0.80;
         gmax=0.97;
      }
      if(tau>=0.6 && tau<0.8) {
         gmin=0.60;
         gmax=0.95;
      }
      if(tau>=0.8) {
         gmin=0.02;
         gmax=0.80;
      }
      rmin=integral(gmin,tau);  /* Comprueba si la raz esta dentro   */
      rmax=integral(gmax,tau);  /* dentro del intervalo elegido       */
      if(rmin*rmax>0)  printf("Raz fuera del intervalo\n");
      while(rps>RPS) {
         i++;
         if(i==JMAX) {
            printf("Demasiadas iteraciones\n");
            exit(1);
         }
         gmitad=0.5*(gmax+gmin);
         rmin=integral(gmin,tau);
         rmitad=integral(gmitad,tau);
         if(rmin*rmitad<0) gmax=gmitad;
         else gmin=gmitad;
         rps=gmax-gmin;
      }
      gmitad=0.5*(gmax+gmin);
      fprintf(out,"%4.3lf   %8.6lf\n",tau,gmitad);
   }
   fclose(out);
}
